
/* 
 *  
 *  CO2 Ampel Projekt des Verstehbahnhof
 *  https://www.verstehbahnhof.de
 *  kontakt@verstehbahnhof.de
 *  Twitter @verstehbahnhof
 *  
 *  v0.2
 *  
 *  Der Sketch verwendet
 *    - die MHZ19 Library von Jonathan Dempsey
 *    - die NeoPixel Library von Adafruit
 *    
 *  Features v0.2
 *    - Setzen von 3 Schwellwerten: gut, mittel, schlecht
 *    - Anzeige der Aufwärmphase des Sensors
 *    - Auslesen des CO2 Sensors
 *    - Setzen der Farbe eines 12bit NeoPixel Rings entsprechend
 *      definierter Schwellwerte
 *  
 *  Todo v0.3
 *    - Fünf-maliges Blinken beim Farbübergang zur besseren
 *      Visualisierung
 *    - WiFi und MQTT Integration zum Übermitteln der Daten
 *  
 *  Hardware
 *    - MH-Z19B CO2 Sensor
 *    - 12bit NeoPixel Ring
 *    - Wemos D1 Mini Microcontroller
 *    
 *  Messwerte, Genauigkeit et al
 *    Der MH-Z19B liefert Messwerte die auf +/- 50ppm genau sind.
 *    Ser Sensor misst Werte gegen angenommene 400ppm CO2 in der Luft.
 *    Beim Start dieses Sketches wird eine Kalibrierung des Sensors
 *    durchgeführt und angenommen das ein 400ppm Zustand herrscht.
 *    Räume sollten somit zum Beginn der Messung gut gelüftet sein.
 *     
 */

#include <Arduino.h>
#include "MHZ19.h"                                        
#include <SoftwareSerial.h>

#include <Adafruit_NeoPixel.h>
#ifdef __AVR__
 #include <avr/power.h> // Required for 16 MHz Adafruit Trinket
#endif


// CO2 Sensor
#define BAUDRATE 9600                                      // Der MH-Z19B verwendet 9600 Baud
#define RX_PIN D7                                          // RX Pin auf dem D1 Mini, verbunden zu TX des MH-Z19B
#define TX_PIN D8                                          // TX Pin auf dem D1 Mini, verbunden zu RX des MH-Z19B

// CO2 Schwellwerte
int co2gut = 500;
int co2mittel = 800;
int co2schlecht = 1100;

// 12er NeoPixel Ring
#define LED_COUNT 12
#define LED_PIN   D5

Adafruit_NeoPixel strip(LED_COUNT, LED_PIN, NEO_GRB + NEO_KHZ800);

MHZ19 myMHZ19;                                             // Constructor for library

SoftwareSerial mySerial(RX_PIN, TX_PIN);                  
unsigned long messLaufzeit = 0;

void setup()
{
    /* 
     * Start der seriellen Kommunikation
     */
     
    Serial.begin(9600);                                     // Device to serial monitor feedback
    mySerial.begin(BAUDRATE);                               // (Uno example) device to MH-Z19 serial start   
    myMHZ19.begin(mySerial);                                // *Serial(Stream) refence must be passed to library begin(). 

    /*
     * Autokalibrierung des Sensors beim Start (abschalten autoCalibration(false))
     */
     
    myMHZ19.autoCalibration();

    /*                 
     *  
     */
     
    #if defined(__AVR_ATtiny85__) && (F_CPU == 16000000)
      clock_prescale_set(clock_div_1);
    #endif

    /*
     * Initalisierung des LED Rings
     */
     
    strip.begin();           // INITIALIZE NeoPixel strip object (REQUIRED)
    strip.show();            // Turn OFF all pixels ASAP
    strip.setBrightness(200); // Set BRIGHTNESS to about 1/5 (max = 255)
}

void loop()
{
    /*
     * Der Sensor hat eine Aufwärmzeit von 180 Sekunden bevor er Messwerte liefert. Solange lassen wir
     * einen Regenbogen durch den Ring laufen. 
     */
     
    if (millis() < 180000)
    {
      rainbow(10);
    }

    /*
     *  Nach der Aufwärmzeit von 180 Sekunden läuft diese Schleife alle fünf Sekunden,
     *  fragt Sensorwerte ab, vergleicht diese mit den eingestellten Schwellwerten und
     *  setzt entsprechend die Farbe der Ampel.
     */

    else if (millis() - messLaufZeit >= 5000)
    {
      
        /*
         * Wir definieren eine Variable CO2 und weisen dieser das ausgelesene CO2-Level vom Sensor zu 
         */
         
        int CO2;
        CO2 = myMHZ19.getCO2();                             // Request CO2 (as ppm)

        /*
         * Wir schreiben uns den CO2 Wert in die serielle Konsole. Die Temperatur nehmen wir gleich mit,
         * gibt ja gratis dazu. 
         */
        
        Serial.print("CO2 (in ppm): ");                      
        Serial.println(CO2);                                
        int8_t Temp;
        Temp = myMHZ19.getTemperature();                     // Request Temperature (as Celsius)
        Serial.print("Temperatur (C): ");                  
        Serial.println(Temp);

        /*
         * Je nach CO2 Level ändern wir die LED-Farbe 
         */
         
        if (CO2 <= co2gut) {                             
          colorWipe(strip.Color(  0, 255,   0), 200);
        }
        else if ((CO2 > co2gut) && (CO2 <= co2mittel)) {
          colorWipe(strip.Color(  255, 255,   0), 200);
        }
        else if ((CO2 > co2mittel) && (CO2 <= co2schlecht)) {
          colorWipe(strip.Color(  255, 165,   0), 200);
        }
        else if (CO2 > co2schlecht) {
          colorWipe(strip.Color(  255, 0,   0), 200);
        }
        messLaufzeit = millis();
    }
}


/* 
 * Zwei Anzeigemodi aus dem NeoPixel Beispielcode. 
 */
 
void colorWipe(uint32_t color, int wait) {
  for(int i=0; i<strip.numPixels(); i++) { // For each pixel in strip...
    strip.setPixelColor(i, color);         //  Set pixel's color (in RAM)
    strip.show();                          //  Update strip to match
    delay(wait);                           //  Pause for a moment
  }
}

void rainbow(int wait) {
  for(long firstPixelHue = 0; firstPixelHue < 5*65536; firstPixelHue += 256) {
    for(int i=0; i<strip.numPixels(); i++) {
      int pixelHue = firstPixelHue + (i * 65536L / strip.numPixels());
      strip.setPixelColor(i, strip.gamma32(strip.ColorHSV(pixelHue)));
    }
    strip.show();
    delay(wait);
  }
}
